TIMESTAMP = 'timestamp'
ICAO = 'icao'

LAST_UPDATE = 'last_update'
MSG_TYPE = 'msg_type'

SESSION = 'session'
SESSION_HASH = 'session_hash'
HOST = 'host'
PORT = 'port'
START = 'start'
STOP = 'stop'
DURATION = 'duration'

IDENT = 'ident'
CALLSIGN = 'callsign'
TYPECODE = 'typecode'
CATEGORY = 'category'

VELOCITY = 'velocity'
SPEED = 'speed'
SPEED_TYPE = 'speed_type'
VERTICAL_SPEED = 'vertical_speed'
VERTICAL_SPEED_SRC = 'vertical_speed_src'
ANGLE = 'angle'
ANGLE_SRC = 'angle_src'

POSITION = 'position'
LATITUDE = 'latitude'
LONGITUDE = 'longitude'
ALTITUDE = 'altitude'
ALTITUDE_SRC = 'altitude_src'